rawset(_G, "FindEmeraldColor", function(p)
	local super_table = nil
	if MasterEmerald.Transformations[p.mo.skin] then
		super_table = MasterEmerald.Transformations[p.mo.skin]
	end
	local color = p.masteremerald.supercolor
	if color == -1 then
		if super_table and super_table.form then
			color = super_table.form.color
		else
			color = 0
		end
	end
	return color
end)

----------------------------
-- Sonic 1 Super Sparkles --
----------------------------
local flameOn = {"blaze", "kiryu", "majima", "saejima", "eggman"}
addHook("MobjThinker", function(mo)
	if not (mo and mo.valid) return end
	if not (mo.target and mo.target.valid and mo.target.player) then return end
	local p = mo.target.player
	local pmo = mo.target
	if p.masteremerald.super then
		local flames = false
		for _,v in ipairs(flameOn)
			if pmo.skin == v then
				flames = true
				break
			end
		end

		local flameScale = {
			3*mo.scale/4,
			mo.scale,
			5*mo.scale/4,
			3*mo.scale/2
		}

		if flames then
			local fire = P_SpawnMobj(mo.x, mo.y, mo.z, MT_THOK) --Goddammit.
			fire.target = pmo
			fire.scale = flameScale[P_RandomRange(1, #flameScale)]
			fire.momz = P_RandomRange(1, 2)*mo.scale
			fire.scalespeed = mo.scale/12
			fire.destscale = 1
			fire.fuse = TICRATE
			if (mo and mo.valid) then
				P_RemoveMobj(mo)
				return -- Go no further, let the THOK MobjThinker handle the rest.
			end
		end
		
		if (mo.tics & 1) then
			mo.flags2 = $ | MF2_DONTDRAW
		else
			mo.flags2 = $ & ~MF2_DONTDRAW
		end
	else
		return
	end
end, MT_SPARKLETRAIL)

addHook("MobjThinker", function(mo)
	if not (mo and mo.valid) then return end
	if not (mo.target and mo.target.valid and mo.target.player) then return end
	local p = mo.target.player
	local pmo = mo.target
	
	if p.masteremerald.super then
		local flames = false
		for _,v in ipairs(flameOn)
			if pmo.skin == v then
				flames = true
				break
			end
		end
		if flames then
			mo.state = S_SUPERFORMHEAT
			mo.frame = FF_FULLBRIGHT|FF_TRANS30
		end
		
		
		local color = FindEmeraldColor(p)
		local color_table =  MasterEmerald.SuperColorList[color]
		if pmo.skin == "eggman" then
			-- Blue flames for Super Eggman
			color_table =  MasterEmerald.SuperColorList[4]
		end
		mo.color = color_table[((leveltime / 3) % #color_table)+1]
		mo.colorized = true
		
		if (mo.fuse & 1) then
			mo.flags2 = $ | MF2_DONTDRAW
		else
			mo.flags2 = $ & ~MF2_DONTDRAW
		end
	end
end, MT_THOK)
--------------------------------------------
-- Super Color / Sprites and Custom Music --
--------------------------------------------
local function SuperFormThinker()
	if leveltime < 2 then return end
	
	for p in players.iterate
		if not (p.mo and p.mo.valid) or p.spectator then continue end
	
		if p.masteremerald.super then
			local super_table = nil
			if MasterEmerald.Transformations[p.mo.skin] then
				super_table = MasterEmerald.Transformations[p.mo.skin]
			end
			-- Super Color
			local color = FindEmeraldColor(p)
			local color_table =  MasterEmerald.SuperColorList[color]
			p.mo.color = color_table[((leveltime / 3) % #color_table)+1]
			p.mo.colorized = false
			-- Custom Super Stuff
			if super_table and super_table.form then
				-- Custom Super Sprites
				p.masteremerald.superrank = super_table.form.superrank
				if super_table.form.sprite then
					p.mo.sprite = super_table.form.sprite.super_sprite
				end
			
				-- Custom Super Music
				if not p.mo.emtrans 
					if super_table.form.music then
						if type(super_table.form.music) == "string" then
							S_ChangeMusic(super_table.form.music, true, p)
						elseif type(super_table.form.music) == "table" then
							local index = P_RandomRange(1,#super_table.form.music)
							S_ChangeMusic(super_table.form.music[index], true, p)
						end
					else
						S_ChangeMusic("emsupr", true, p)
					end
					p.mo.emtrans = true
				end
			else
				if p.masteremerald.superrank then
					p.masteremerald.superrank = nil
				end
			end
		else
			if p.mo.emtrans
				p.mo.emtrans = nil
			end
		end
	end
end
addHook("ThinkFrame", SuperFormThinker)

addHook("MobjThinker", function(mo)
	if not (mo and mo.valid) then return end
	
	if (mo.target and mo.target.valid)
	and (mo.target.target and mo.target.target.valid and mo.target.target.player)
	and not mo.emsignset then
		-- print("Sign linked to player.")
		-- print("Did player finish in Super Form? "..mo.target.target.player.emsuperfinish)
		if mo.target.target.player.emsuperfinish
			mo.emsupersign = true
			-- print("Super Finish!")
		end
		mo.emsignset = 1
	end

	if mo.emsupersign
	and not mo.emsupergoalset then
		-- Keep Sign color the same even if you change colors after the 
		-- sign spawns
		mo.emsupercolor = FindEmeraldColor(mo.target.target.player)
		mo.emsupergoalset = 1 -- This keeps the goalpost the same in case you switch characters while a race is still underway.
	end

	if mo.emsupergoalset then
		local color_table =  MasterEmerald.SuperColorList[mo.emsupercolor]
		mo.color = color_table[((leveltime / 3) % #color_table)+1]
	end
	
	if mo.state == S_PLAY_SIGN
	and (mo.target.target and mo.target.target.player 
	and MasterEmerald.Transformations
	and MasterEmerald.Transformations[mo.target.target.skin] 
	and MasterEmerald.Transformations[mo.target.target.skin].form
	and MasterEmerald.Transformations[mo.target.target.skin].form.sprite)
		if mo.emsupergoalset and mo.target.target.player.emsuperfinish then
			mo.state = MasterEmerald.Transformations[mo.target.target.skin].form.sprite.super_sign
		end
	end
end, MT_OVERLAY)